@@ -25,7 +25,7 @@
 
 #include "arrow/array/data.h"
 #include "arrow/compute/api_vector.h"
-#include "arrow/compute/kernels/common.h"
+#include "arrow/compute/kernels/common_internal.h"
 #include "arrow/compute/kernels/util_internal.h"
 #include "arrow/compute/kernels/vector_sort_internal.h"
 #include "arrow/type_traits.h"
@@ -459,7 +459,8 @@ Status ArraySortIndicesChunked(KernelContext* ctx, const ExecBatch& batch, Datum
   std::iota(out_begin, out_end, 0);
   return SortChunkedArray(ctx->exec_context(), out_begin, out_end,
                           *batch[0].chunked_array(), options.order,
-                          options.null_placement);
+                          options.null_placement)
+      .status();
 }
 
 template <template <typename...> class ExecTemplate>
