@@ -31,7 +31,7 @@
 #include <vector>
 
 #include "arrow/compute/api_aggregate.h"
-#include "arrow/compute/exec/expression.h"
+#include "arrow/compute/expression.h"
 #include "arrow/engine/substrait/type_fwd.h"
 #include "arrow/engine/substrait/visibility.h"
 #include "arrow/result.h"
@@ -67,12 +67,18 @@ constexpr const char* kSubstraitAggregateGenericFunctionsUri =
     "https://github.com/substrait-io/substrait/blob/main/extensions/"
     "functions_aggregate_generic.yaml";
 
-struct Id {
+/// If a function call contains this URI then the function is looked up
+/// in the registry directly, all arguments are mapped as value arguments,
+/// and any options are ignored.
+constexpr const char* kArrowSimpleExtensionFunctionsUri =
+    "urn:arrow:substrait_simple_extension_function";
+
+struct ARROW_ENGINE_EXPORT Id {
   std::string_view uri, name;
   bool empty() const { return uri.empty() && name.empty(); }
   std::string ToString() const;
 };
-struct IdHashEq {
+struct ARROW_ENGINE_EXPORT IdHashEq {
   size_t operator()(Id id) const;
   bool operator()(Id l, Id r) const;
 };
@@ -84,7 +90,7 @@ struct IdHashEq {
 /// convert a plan from Substrait to Arrow we need to copy these strings out of
 /// the Substrait buffer and into owned storage.  This class serves as that owned
 /// storage.
-class IdStorage {
+class ARROW_ENGINE_EXPORT IdStorage {
  public:
   virtual ~IdStorage() = default;
   /// \brief Get an equivalent id pointing into this storage
@@ -112,7 +118,7 @@ class IdStorage {
 /// Substrait call expressions contain a list of arguments which can either
 /// be enum arguments (which are serialized as strings), value arguments (which)
 /// are Arrow expressions, or type arguments (not yet implemented)
-class SubstraitCall {
+class ARROW_ENGINE_EXPORT SubstraitCall {
  public:
   SubstraitCall(Id id, std::shared_ptr<DataType> output_type, bool output_nullable,
                 bool is_hash = false)
@@ -136,6 +142,7 @@ class SubstraitCall {
       std::string_view option_name) const;
   void SetOption(std::string_view option_name,
                  const std::vector<std::string_view>& option_preferences);
+  bool HasOptions() const;
   int size() const { return size_; }
 
  private:
