@@ -635,6 +635,28 @@ std::shared_ptr<Array> RandomArrayGenerator::Map(const std::shared_ptr<Array>& k
   return *::arrow::MapArray::FromArrays(offsets, keys, items);
 }
 
+std::shared_ptr<Array> RandomArrayGenerator::RunEndEncoded(
+    std::shared_ptr<DataType> value_type, int64_t logical_size, double null_probability) {
+  Int32Builder run_ends_builder;
+  pcg32_fast rng(seed());
+
+  DCHECK_LE(logical_size, std::numeric_limits<int32_t>::max());
+
+  std::uniform_int_distribution<int64_t> distribution(1, 100);
+  int64_t current_end = 0;
+  while (current_end < logical_size) {
+    current_end += distribution(rng);
+    current_end = std::min(current_end, logical_size);
+    ARROW_CHECK_OK(run_ends_builder.Append(static_cast<int32_t>(current_end)));
+  }
+
+  std::shared_ptr<Array> run_ends = *run_ends_builder.Finish();
+  std::shared_ptr<Array> values =
+      ArrayOf(std::move(value_type), run_ends->length(), null_probability);
+
+  return RunEndEncodedArray::Make(logical_size, run_ends, values).ValueOrDie();
+}
+
 std::shared_ptr<Array> RandomArrayGenerator::SparseUnion(const ArrayVector& fields,
                                                          int64_t size, int64_t alignment,
                                                          MemoryPool* memory_pool) {
@@ -771,7 +793,7 @@ std::shared_ptr<Array> RandomArrayGenerator::ArrayOf(const Field& field, int64_t
                 values_length, alignment, memory_pool);                              \
     const auto offsets = OffsetsFromLengthsArray(lengths.get(), force_empty_nulls,   \
                                                  alignment, memory_pool);            \
-    return *ARRAY_TYPE::FromArrays(*offsets, *values);                               \
+    return *ARRAY_TYPE::FromArrays(field.type(), *offsets, *values);                 \
   }
 
   const double null_probability =
@@ -894,17 +916,22 @@ std::shared_ptr<Array> RandomArrayGenerator::ArrayOf(const Field& field, int64_t
 
     case Type::type::STRUCT: {
       ArrayVector child_arrays(field.type()->num_fields());
-      std::vector<std::string> field_names;
+      FieldVector child_fields(field.type()->num_fields());
       for (int i = 0; i < field.type()->num_fields(); i++) {
         const auto& child_field = field.type()->field(i);
         child_arrays[i] = ArrayOf(*child_field, length, alignment, memory_pool);
-        field_names.push_back(child_field->name());
+        child_fields[i] = child_field;
       }
       return *StructArray::Make(
-          child_arrays, field_names,
+          child_arrays, child_fields,
           NullBitmap(length, null_probability, alignment, memory_pool));
     }
 
+    case Type::type::RUN_END_ENCODED: {
+      auto* ree_type = internal::checked_cast<RunEndEncodedType*>(field.type().get());
+      return RunEndEncoded(ree_type->value_type(), length, null_probability);
+    }
+
     case Type::type::SPARSE_UNION:
     case Type::type::DENSE_UNION: {
       ArrayVector child_arrays(field.type()->num_fields());
