@@ -135,11 +135,11 @@ std::basic_string<CharT> ReplaceChars(std::basic_string<CharT> s, CharT find, Ch
   return s;
 }
 
-Result<NativePathString> StringToNative(const std::string& s) {
+Result<NativePathString> StringToNative(std::string_view s) {
 #if _WIN32
   return ::arrow::util::UTF8ToWideString(s);
 #else
-  return s;
+  return std::string(s);
 #endif
 }
 
@@ -193,7 +193,7 @@ NativePathString NativeParent(const NativePathString& s) {
   }
 }
 
-Status ValidatePath(const std::string& s) {
+Status ValidatePath(std::string_view s) {
   if (s.find_first_of('\0') != std::string::npos) {
     return Status::Invalid("Embedded NUL char in path: '", s, "'");
   }
@@ -589,7 +589,7 @@ Result<PlatformFilename> PlatformFilename::Real() const {
   return PlatformFilename(std::move(real));
 }
 
-Result<PlatformFilename> PlatformFilename::FromString(const std::string& file_name) {
+Result<PlatformFilename> PlatformFilename::FromString(std::string_view file_name) {
   RETURN_NOT_OK(ValidatePath(file_name));
   ARROW_ASSIGN_OR_RAISE(auto ns, StringToNative(file_name));
   return PlatformFilename(std::move(ns));
@@ -603,8 +603,9 @@ PlatformFilename PlatformFilename::Join(const PlatformFilename& child) const {
   }
 }
 
-Result<PlatformFilename> PlatformFilename::Join(const std::string& child_name) const {
-  ARROW_ASSIGN_OR_RAISE(auto child, PlatformFilename::FromString(child_name));
+Result<PlatformFilename> PlatformFilename::Join(std::string_view child_name) const {
+  ARROW_ASSIGN_OR_RAISE(auto child,
+                        PlatformFilename::FromString(std::string(child_name)));
   return Join(child);
 }
 
